#region usings
using System;
using System.ComponentModel.Composition;
using System.Collections.Generic;

using VVVV.PluginInterfaces.V1;
using VVVV.PluginInterfaces.V2;
using VVVV.Utils.VColor;
using VVVV.Utils.VMath;

using VVVV.Core.Logging;
#endregion usings

namespace VVVV.Nodes
{
	#region PluginInfo
	[PluginInfo(Name = "Sender", Category = "Event", Help = "Basic template with one value in/out", Tags = "", AutoEvaluate=true)]
	#endregion PluginInfo
	public class EventSenderNode : IPluginEvaluate
	{
		#region fields & pins
		[Input("Data", DefaultValue = 0.0)]
		IDiffSpread<double> FData;

//		[Input("Name", DefaultString = "Pin")]
		ISpread<string> FName;

//		[Input("BinSize", DefaultValue = 1.0)]
		ISpread<int> FBinSize;

		[Input("Create", DefaultValue = 0.0)]
		ISpread<bool> FCreate;

		[Output("Initialized")]
		ISpread<bool> FInitialized;

		[Import()]
		ILogger FLogger;

		[Import()]
		IPluginHost FHost;
		
		Dictionary<String, IValueOut> pins = new Dictionary<String, IValueOut>();
		Dictionary<String, int> binSize = new Dictionary<String, int>();
		bool initialized = false;
		
		#endregion fields & pins
		
		[ImportingConstructor]
		public EventSenderNode([Input("Name")] ISpread<string> name, [Input("BinSize")] ISpread<int> binsize ) {
			FName = name;
			FBinSize = binsize;
			
			Create();
		}
		
		public void Reset() {
			foreach (string key in pins.Keys) {
				FHost.DeletePin(pins[key]);
			}
			pins.Clear();
			
		}
		
		public void Create() {
			initialized = true;
			
			IValueOut pin;
			Dictionary<String, IValueOut> newPins = new Dictionary<String, IValueOut>();
			
			for (int i=0;i<FName.SliceCount;i++) {
				string key = FName[i];
				
				if (pins.ContainsKey(key)) {
					if (binSize[key] != FBinSize[i]) {
						pin = pins[key];
						newPins.Add(key, pin);
						binSize[key] = FBinSize[i];
					} else newPins.Add(key, pins[key]);
					
					newPins[key].Order = i;
					pins.Remove(key);
				} else {
					FHost.CreateValueOutput(key, 1, new string[] {""}, TSliceMode.Dynamic, TPinVisibility.True, out pin);
					newPins.Add(key, pin);
					binSize.Add(key, FBinSize[i]);
					pin.Order = i;
				}
			}
			
			foreach (string key in pins.Keys) {
				FHost.DeletePin(pins[key]);
				binSize.Remove(key);
			}
			
			pins = newPins;
		}
		
		public void Evaluate(int SpreadMax)
		{
			if (!initialized) {
//				Create();
			}
			FInitialized.SliceCount = 1;
			FInitialized[0] = initialized;
			
			if (FCreate[0]) {
				Create();
			}

			if (!FData.IsChanged) return;
			
			int totalCount = 0;
			foreach (string key in pins.Keys) {
				int count = binSize[key];
				
				pins[key].SliceCount = count;
				for (int i = 0;i<count;i++) 
					((IValueOut)pins[key]).SetValue(i, FData[totalCount+i]);
				totalCount+=count;
			}
		}
	}
}
